@@ -1,31 +1,65 @@
 from __future__ import generators
 
-import time
+import os
+import sys
 import operator
-try:
-    import ctypes
-except ImportError:
-    ctypes = None
 import importlib
 import logging
+import threading
+import traceback
+import warnings
 
-from datetime import datetime
-from uuid import UUID, uuid4, _uuid_generate_random
 from inspect import getargspec
 from itertools import islice
+from pprint import pprint
 
-from carrot.utils import rpartition
-from dateutil.parser import parse as parse_iso8601
+from kombu.utils import gen_unique_id, rpartition, cached_property
 
-from celery.utils.compat import all, any, defaultdict
-from celery.utils.timeutils import timedelta_seconds        # was here before
-from celery.utils.functional import curry
+from celery.utils.compat import StringIO
+from celery.utils.functional import partial, wraps
 
 
 LOG_LEVELS = dict(logging._levelNames)
 LOG_LEVELS["FATAL"] = logging.FATAL
 LOG_LEVELS[logging.FATAL] = "FATAL"
 
+PENDING_DEPRECATION_FMT = """
+    %(description)s is scheduled for deprecation in \
+    version %(deprecation)s and removal in version v%(removal)s. \
+    %(alternative)s
+"""
+
+DEPRECATION_FMT = """
+    %(description)s is deprecated and scheduled for removal in
+    version %(removal)s. %(alternative)s
+"""
+
+
+def deprecated(description=None, deprecation=None, removal=None,
+        alternative=None):
+
+    def _inner(fun):
+
+        @wraps(fun)
+        def __inner(*args, **kwargs):
+            ctx = {"description": description or get_full_cls_name(fun),
+                   "deprecation": deprecation, "removal": removal,
+                   "alternative": alternative}
+            if deprecation is not None:
+                w = PendingDeprecationWarning(PENDING_DEPRECATION_FMT % ctx)
+            else:
+                w = DeprecationWarning(DEPRECATION_FMT % ctx)
+            warnings.warn(w)
+            return fun(*args, **kwargs)
+        return __inner
+    return _inner
+
+
+def lpmerge(L, R):
+    """Left precedent dictionary merge.  Keeps values from `l`, if the value
+    in `r` is :const:`None`."""
+    return dict(L, **dict((k, v) for k, v in R.iteritems() if v is not None))
+
 
 class promise(object):
     """A promise.
@@ -61,6 +95,9 @@ class promise(object):
             return -cmp(rhs, self())
         return cmp(self(), rhs)
 
+    def __eq__(self, rhs):
+        return self() == rhs
+
     def __deepcopy__(self, memo):
         memo[id(self)] = self
         return self
@@ -107,15 +144,6 @@ def noop(*args, **kwargs):
     pass
 
 
-def maybe_iso8601(dt):
-    """``Either datetime | str -> datetime or None -> None``"""
-    if not dt:
-        return
-    if isinstance(dt, datetime):
-        return dt
-    return parse_iso8601(dt)
-
-
 def kwdict(kwargs):
     """Make sure keyword arguments are not in unicode.
 
@@ -128,7 +156,7 @@ def kwdict(kwargs):
 
 
 def first(predicate, iterable):
-    """Returns the first element in ``iterable`` that ``predicate`` returns a
+    """Returns the first element in `iterable` that `predicate` returns a
     :const:`True` value for."""
     for item in iterable:
         if predicate(item):
@@ -155,7 +183,7 @@ def firstmethod(method):
 
 
 def chunks(it, n):
-    """Split an iterator into chunks with ``n`` elements each.
+    """Split an iterator into chunks with `n` elements each.
 
     Examples
 
@@ -174,20 +202,6 @@ def chunks(it, n):
         yield [first] + list(islice(it, n - 1))
 
 
-def gen_unique_id():
-    """Generate a unique id, having - hopefully - a very small chance of
-    collission.
-
-    For now this is provided by :func:`uuid.uuid4`.
-    """
-    # Workaround for http://bugs.python.org/issue4607
-    if ctypes and _uuid_generate_random:
-        buffer = ctypes.create_string_buffer(16)
-        _uuid_generate_random(buffer)
-        return str(UUID(bytes=buffer.raw))
-    return str(uuid4())
-
-
 def padlist(container, size, default=None):
     """Pad list with default elements.
 
@@ -213,12 +227,6 @@ def is_iterable(obj):
     return True
 
 
-def mitemgetter(*items):
-    """Like :func:`operator.itemgetter` but returns :const:`None`
-    on missing items instead of raising :exc:`KeyError`."""
-    return lambda container: map(container.get, items)
-
-
 def mattrgetter(*attrs):
     """Like :func:`operator.itemgetter` but returns :const:`None` on missing
     attributes instead of raising :exc:`AttributeError`."""
@@ -232,64 +240,12 @@ def get_full_cls_name(cls):
                      cls.__name__])
 
 
-def repeatlast(it):
-    """Iterate over all elements in the iterator, and when its exhausted
-    yield the last value infinitely."""
-    for item in it:
-        yield item
-    while 1:                                            # pragma: no cover
-        yield item
-
-
-def retry_over_time(fun, catch, args=[], kwargs={}, errback=noop,
-        max_retries=None, interval_start=2, interval_step=2, interval_max=30):
-    """Retry the function over and over until max retries is exceeded.
-
-    For each retry we sleep a for a while before we try again, this interval
-    is increased for every retry until the max seconds is reached.
-
-    :param fun: The function to try
-    :param catch: Exceptions to catch, can be either tuple or a single
-        exception class.
-    :keyword args: Positional arguments passed on to the function.
-    :keyword kwargs: Keyword arguments passed on to the function.
-    :keyword errback: Callback for when an exception in ``catch`` is raised.
-        The callback must take two arguments: ``exc`` and ``interval``, where
-        ``exc`` is the exception instance, and ``interval`` is the time in
-        seconds to sleep next..
-    :keyword max_retries: Maximum number of retries before we give up.
-        If this is not set, we will retry forever.
-    :keyword interval_start: How long (in seconds) we start sleeping between
-        retries.
-    :keyword interval_step: By how much the interval is increased for each
-        retry.
-    :keyword interval_max: Maximum number of seconds to sleep between retries.
-
-    """
-    retries = 0
-    interval_range = xrange(interval_start,
-                            interval_max + interval_start,
-                            interval_step)
-
-    for interval in repeatlast(interval_range):
-        try:
-            retval = fun(*args, **kwargs)
-        except catch, exc:
-            if max_retries and retries > max_retries:
-                raise
-            errback(exc, interval)
-            retries += 1
-            time.sleep(interval)
-        else:
-            return retval
-
-
 def fun_takes_kwargs(fun, kwlist=[]):
     """With a function, and a list of keyword arguments, returns arguments
     in the list which the function takes.
 
-    If the object has an ``argspec`` attribute that is used instead
-    of using the :meth:`inspect.getargspec`` introspection.
+    If the object has an `argspec` attribute that is used instead
+    of using the :meth:`inspect.getargspec` introspection.
 
     :param fun: The function to inspect arguments of.
     :param kwlist: The list of keyword arguments.
@@ -310,10 +266,10 @@ def fun_takes_kwargs(fun, kwlist=[]):
     args, _varargs, keywords, _defaults = argspec
     if keywords != None:
         return kwlist
-    return filter(curry(operator.contains, args), kwlist)
+    return filter(partial(operator.contains, args), kwlist)
 
 
-def get_cls_by_name(name, aliases={}):
+def get_cls_by_name(name, aliases={}, imp=None):
     """Get class by name.
 
     The name should be the full dot-separated path to the class::
@@ -325,7 +281,7 @@ def get_cls_by_name(name, aliases={}):
         celery.concurrency.processes.TaskPool
                                     ^- class name
 
-    If ``aliases`` is provided, a dict containing short name/long name
+    If `aliases` is provided, a dict containing short name/long name
     mappings, the name is looked up in the aliases first.
 
     Examples:
@@ -343,15 +299,22 @@ def get_cls_by_name(name, aliases={}):
         True
 
     """
+    if imp is None:
+        imp = importlib.import_module
 
     if not isinstance(name, basestring):
-        return name                                     # already a class
+        return name                                 # already a class
 
     name = aliases.get(name) or name
     module_name, _, cls_name = rpartition(name, ".")
-    module = importlib.import_module(module_name)
+    try:
+        module = imp(module_name)
+    except ValueError, exc:
+        raise ValueError("Couldn't import %r: %s" % (name, exc))
     return getattr(module, cls_name)
 
+get_symbol_by_name = get_cls_by_name
+
 
 def instantiate(name, *args, **kwargs):
     """Instantiate class by name.
@@ -382,6 +345,69 @@ def abbrtask(S, max):
         return "???"
     if len(S) > max:
         module, _, cls = rpartition(S, ".")
-        module = abbr(module, max - len(cls), False)
+        module = abbr(module, max - len(cls) - 3, False)
         return module + "[.]" + cls
     return S
+
+
+def isatty(fh):
+    # Fixes bug with mod_wsgi:
+    #   mod_wsgi.Log object has no attribute isatty.
+    return getattr(fh, "isatty", None) and fh.isatty()
+
+
+def textindent(t, indent=0):
+        """Indent text."""
+        return "\n".join(" " * indent + p for p in t.split("\n"))
+
+
+def import_from_cwd(module, imp=None):
+    """Import module, but make sure it finds modules
+    located in the current directory.
+
+    Modules located in the current directory has
+    precedence over modules located in `sys.path`.
+    """
+    if imp is None:
+        imp = importlib.import_module
+    cwd = os.getcwd()
+    if cwd in sys.path:
+        return imp(module)
+    sys.path.insert(0, cwd)
+    try:
+        return imp(module)
+    finally:
+        try:
+            sys.path.remove(cwd)
+        except ValueError:
+            pass
+
+
+def cry():
+    """Return stacktrace of all active threads.
+
+    From https://gist.github.com/737056
+
+    """
+    tmap = {}
+    main_thread = None
+    # get a map of threads by their ID so we can print their names
+    # during the traceback dump
+    for t in threading.enumerate():
+        if getattr(t, "ident", None):
+            tmap[t.ident] = t
+        else:
+            main_thread = t
+
+    out = StringIO()
+    for tid, frame in sys._current_frames().iteritems():
+        thread = tmap.get(tid, main_thread)
+        out.write("%s\n" % (thread.getName(), ))
+        out.write("=================================================\n")
+        traceback.print_stack(frame, file=out)
+        out.write("=================================================\n")
+        out.write("LOCAL VARIABLES\n")
+        out.write("=================================================\n")
+        pprint(frame.f_locals, stream=out)
+        out.write("\n\n")
+    return out.getvalue()
