@@ -515,9 +515,9 @@ else:
             This handler is based on a suggestion and patch by Chad J.
             Schroeder.
             """
-            def __init__(self, filename, mode='a', encoding=None, delay=0):
-                logging.FileHandler.__init__(self, filename, mode,
-                                             encoding, delay)
+            def __init__(self, *args, **kwargs):
+                logging.FileHandler.__init__(self, *args, **kwargs)
+
                 if not os.path.exists(self.baseFilename):
                     self.dev, self.ino = -1, -1
                 else:
